% DL examples provided in:
% https://de.mathworks.com/help/nnet/ref/trainnetwork.html
% https://de.mathworks.com/help/vision/examples/object-detection-using-deep-learning.html

% load tha data and provide the labels
patchSonars = imageDatastore('DL_Data', ...
        'IncludeSubfolders',true,'LabelSource','foldernames');

% 4/5 of data is used for training, 1/5 for the test
trainingNumFiles = 4/5;

% split the data in training adn test set
[trainPatches,testPatches] = splitEachLabel(patchSonars, ...
				trainingNumFiles,'randomize');
length(trainPatches.Labels)
length(testPatches.Labels)
% define CNN 
layers = [imageInputLayer([50 50 1])
          convolution2dLayer(5,20)
          reluLayer
          maxPooling2dLayer(2,'Stride',2)
          fullyConnectedLayer(2)
          softmaxLayer
          classificationLayer()]; 
% define hyperparameter - default values      
options = trainingOptions('sgdm','MaxEpochs',50, ...
	'InitialLearnRate',0.001);

% train the netwrok
convnet = trainNetwork(trainPatches,layers,options);    

% evaluate performance 
YTest = classify(convnet,testPatches);
TTest = testPatches.Labels;

TP = sum((TTest == 'Bagger') & (YTest == TTest));
TN = sum((TTest == 'noBagger') & (YTest == TTest));
FP = sum((TTest == 'noBagger') & (YTest == 'Bagger'));
FN = sum((TTest == 'Bagger') & (YTest == 'noBagger'));

accuracy = (TP+TN)/(TP+FP+FN+TN)
precision = TP/(TP+FP)
recall = TP/(TP+FN)
F1_Score = 2*(recall * precision) / (recall + precision)

confidence_interval = 2.58 * sqrt( (accuracy * (1 - accuracy)) / length(YTest))