import pandas as pd
import numpy as np
from sklearn.linear_model import SGDClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import precision_score, recall_score, f1_score, accuracy_score
from sklearn.model_selection import cross_val_predict
from sklearn.metrics import roc_auc_score
from sklearn.metrics import roc_curve
from sklearn.metrics import RocCurveDisplay
import matplotlib.pyplot as plt
from sklearn.model_selection import GridSearchCV
import timeit
from imblearn.metrics import classification_report_imbalanced
from sklearn.metrics import precision_recall_curve

# load data
X = pd.read_csv('Excavator_data.csv')
y = pd.read_csv('Excavator_labels.csv')

# split the data in training and the test set (20% of data are in the test set)
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)
y_train = np.ravel(y_train)
y_test = np.ravel(y_test)

print('x_train  y_train shape  y test shape')
print(X_train.shape, y_train.shape, y_test.shape)

# Test each classifier
# Note: if you would like to optimize hyperparameter remove comments from the appropriate part of the code

# kNN Classifier
from sklearn.neighbors import KNeighborsClassifier
clf = KNeighborsClassifier()
#parameters = {'n_neighbors': [1,3,5]}
#grid_search = GridSearchCV(clf, parameters, cv = 5, scoring = 'accuracy', return_train_score = 'True')
#grid_search.fit(X_train, y_train)
#print('Best parameters kNN:\n', grid_search.best_params_)
#knn_clf = grid_search.best_estimator_

knn_clf = KNeighborsClassifier(n_neighbors=1)


y_train_pred = cross_val_predict(knn_clf, X_train, y_train, cv=3)

# Performance Measures
print('knn_classifer:')
print(' accuracy       precision           recall                  F1             roc_auc                confidence interval' )
print(accuracy_score(y_train, y_train_pred), precision_score(y_train, y_train_pred), recall_score(y_train, y_train_pred), f1_score(y_train, y_train_pred), roc_auc_score(y_train, y_train_pred), 2.58*np.sqrt(accuracy_score(y_train, y_train_pred)*(1- accuracy_score(y_train, y_train_pred))/len(y_train)))

start = timeit.default_timer()
knn_clf.fit(X_train, y_train)
stop = timeit.default_timer()
print('kNN_Classifier Training Time: ', stop - start)

start = timeit.default_timer()
y_pred = knn_clf.predict(X_test)
stop = timeit.default_timer()
print('kNN_Classifier Test Time: ', stop - start)
print(accuracy_score(y_train, y_train_pred), precision_score(y_test, y_pred), recall_score(y_test, y_pred), f1_score(y_test, y_pred), roc_auc_score(y_test, y_pred), 2.58*np.sqrt(accuracy_score(y_test, y_pred)*(1- accuracy_score(y_test, y_pred))/len(y_test)))

print('kNN_classifer: classification report')
print(classification_report_imbalanced(y_test, y_pred))

print('kNN_classifier:std_var')
print(np.std(y_test - y_pred))

#Plot ROC-Curve
prec, recall, _ = precision_recall_curve(y_train, y_train_pred)
plt.plot(recall, prec, label = "kNN Classifier")
#roc_display = RocCurveDisplay(fpr=fpr, tpr=tpr).plot()


# Decision Tree Classifier
from sklearn.tree import DecisionTreeClassifier
#clf = DecisionTreeClassifier(random_state=0)
#clf = DecisionTreeClassifier()
#parameters = {'min_samples_split': [2,10,20], 'min_samples_leaf': [1,10,20]}
#grid_search = GridSearchCV(clf, parameters, cv = 5, scoring = 'accuracy', return_train_score = 'True')
#grid_search.fit(X_train, y_train)
#print('Best parameters DT:\n', grid_search.best_params_)
#dt_clf = grid_search.best_estimator_
dt_clf = DecisionTreeClassifier(min_samples_leaf= 1, min_samples_split= 2, random_state=0)



#dt_clf.fit(X_train, y_train)
y_train_pred = cross_val_predict(dt_clf, X_train, y_train, cv=3)

# Performance Metrics
print('decision_tree_classifer:')
print(' accuracy       precision           recall                  F1             roc_auc                confidence interval' )
print(accuracy_score(y_train, y_train_pred), precision_score(y_train, y_train_pred), recall_score(y_train, y_train_pred), f1_score(y_train, y_train_pred), roc_auc_score(y_train, y_train_pred), 2.58*np.sqrt(accuracy_score(y_train, y_train_pred)*(1- accuracy_score(y_train, y_train_pred))/len(y_train)))

start = timeit.default_timer()
dt_clf.fit(X_train, y_train)
stop = timeit.default_timer()
print('DT_Classifier Training Time: ', stop - start)

start = timeit.default_timer()
y_pred = dt_clf.predict(X_test)
stop = timeit.default_timer()
print('DT_Classifier Test Time: ', stop - start)

print(accuracy_score(y_train, y_train_pred), precision_score(y_test, y_pred), recall_score(y_test, y_pred), f1_score(y_test, y_pred), roc_auc_score(y_test, y_pred), 2.58*np.sqrt(accuracy_score(y_test, y_pred)*(1- accuracy_score(y_test, y_pred))/len(y_test)))

print('DT_classifer: classification report')
print(classification_report_imbalanced(y_test, y_pred))

print('DT_classifier:std_var')
print(np.std(y_test - y_pred))

prec, recall, _ = precision_recall_curve(y_train, y_train_pred)
plt.plot(recall, prec, label = "DecisionTreeClassifier")

# Random Forrest Classifier
from sklearn.ensemble import RandomForestClassifier
rf_clf = RandomForestClassifier(max_depth=100, n_estimators = 100, random_state=0)
#clf = RandomForestClassifier()
#parameters = {'n_estimators': [10,100,200], 'max_depth': [3,10,100]}
#grid_search = GridSearchCV(clf, parameters, cv = 5, scoring = 'accuracy', return_train_score = 'True')
#grid_search.fit(X_train, y_train)
#print('Best parameters Random Forest:\n', grid_search.best_params_)
#rf_clf = grid_search.best_estimator_

y_train_pred = cross_val_predict(rf_clf, X_train, y_train, cv=3)

# Performance Metrics
print('randomForest_classifer:')
print(' accuracy       precision           recall                  F1             roc_auc                confidence interval' )
print(accuracy_score(y_train, y_train_pred), precision_score(y_train, y_train_pred), recall_score(y_train, y_train_pred), f1_score(y_train, y_train_pred), roc_auc_score(y_train, y_train_pred), 2.58*np.sqrt(accuracy_score(y_train, y_train_pred)*(1- accuracy_score(y_train, y_train_pred))/len(y_train)))

start = timeit.default_timer()
rf_clf.fit(X_train, y_train)
stop = timeit.default_timer()
print('RandomForest_Classifier Training Time: ', stop - start)

start = timeit.default_timer()
y_pred = rf_clf.predict(X_test)
stop = timeit.default_timer()
print('RandomForest_Classifier Test Time: ', stop - start)
print(accuracy_score(y_train, y_train_pred), precision_score(y_test, y_pred), recall_score(y_test, y_pred), f1_score(y_test, y_pred), roc_auc_score(y_test, y_pred), 2.58*np.sqrt(accuracy_score(y_test, y_pred)*(1- accuracy_score(y_test, y_pred))/len(y_test)))

print('RandomForestClassifier: classification report')
print(classification_report_imbalanced(y_test, y_pred))

print('RandomForestClassifier:std_var')
print(np.std(y_test - y_pred))

#Plot ROC-Curve

prec, recall, _ = precision_recall_curve(y_train, y_train_pred)
plt.plot(recall, prec, label = "RandomForestClassifier")

# NN Classifier
from sklearn.neural_network import MLPClassifier
nn_clf = MLPClassifier(solver='adam', alpha=1e-5,hidden_layer_sizes=(20, 2), random_state=1)
#nn_clf.fit(X_train, y_train)
#clf = MLPClassifier()
#parameters = {'solver': ['lbfgs','sgd','adam'], 'alpha': [1e-4, 1e-5], 'hidden_layer_sizes': [(5, 2),(10, 2),(20, 2)]}
#grid_search = GridSearchCV(clf, parameters, cv = 5, scoring = 'accuracy', return_train_score = 'True')
#grid_search.fit(X_train, y_train)
#print('Best parameters NN:\n', grid_search.best_params_)

#nn_clf = grid_search.best_estimator_

y_train_pred = cross_val_predict(nn_clf, X_train, y_train, cv=3)

#Performance
print('nn_classifer:')
print(' accuracy       precision           recall                  F1             roc_auc                confidence interval' )
print(accuracy_score(y_train, y_train_pred), precision_score(y_train, y_train_pred), recall_score(y_train, y_train_pred), f1_score(y_train, y_train_pred), roc_auc_score(y_train, y_train_pred), 2.58*np.sqrt(accuracy_score(y_train, y_train_pred)*(1- accuracy_score(y_train, y_train_pred))/len(y_train)))

start = timeit.default_timer()
nn_clf.fit(X_train, y_train)
stop = timeit.default_timer()
print('NN_Classifier Training Time: ', stop - start)

start = timeit.default_timer()
y_pred = nn_clf.predict(X_test)
stop = timeit.default_timer()
print('NN_Classifier Test Time: ', stop - start)
print(accuracy_score(y_train, y_train_pred), precision_score(y_test, y_pred), recall_score(y_test, y_pred), f1_score(y_test, y_pred), roc_auc_score(y_test, y_pred), 2.58*np.sqrt(accuracy_score(y_test, y_pred)*(1- accuracy_score(y_test, y_pred))/len(y_test)))

print('NN_Classifier: classification report')
print(classification_report_imbalanced(y_test, y_pred))
print('NN_Classifier:std_var')
print(np.std(y_test - y_pred))

#Plot ROC-Curve

prec, recall, _ = precision_recall_curve(y_train, y_train_pred)
plt.plot(recall, prec, label = "Full Connected NN Classifier")


# SVM Classifier
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import StandardScaler

from sklearn.svm import SVC

svm_clf = SVC(C = 10, gamma = 'auto')
#parameters = {'C': [ 0.1, 1, 10], 'gamma' : ['scale', 'auto']}
#grid_search = GridSearchCV(clf, parameters, cv = 5, scoring = 'accuracy', return_train_score = 'True')
#grid_search.fit(X_train, y_train)
#print('Best parameters SVM:\n', grid_search.best_params_)

#svm_clf = grid_search.best_estimator_

svm_clf = make_pipeline(StandardScaler(), svm_clf)

y_train_pred = cross_val_predict(svm_clf, X_train, y_train, cv=3)
print('SVM_Classifer:')
print(' accuracy       precision           recall                  F1             roc_auc                confidence interval' )
print(accuracy_score(y_train, y_train_pred), precision_score(y_train, y_train_pred), recall_score(y_train, y_train_pred), f1_score(y_train, y_train_pred), roc_auc_score(y_train, y_train_pred), 2.58*np.sqrt(accuracy_score(y_train, y_train_pred)*(1- accuracy_score(y_train, y_train_pred))/len(y_train)))


start = timeit.default_timer()
svm_clf.fit(X_train, y_train)
stop = timeit.default_timer()
print('SVM_Classifier Training Time: ', stop - start)

start = timeit.default_timer()
y_pred = svm_clf.predict(X_test)
stop = timeit.default_timer()
print('SVM_Classifier Test Time: ', stop - start)
print(accuracy_score(y_train, y_train_pred), precision_score(y_test, y_pred), recall_score(y_test, y_pred), f1_score(y_test, y_pred), roc_auc_score(y_test, y_pred), 2.58*np.sqrt(accuracy_score(y_test, y_pred)*(1- accuracy_score(y_test, y_pred))/len(y_test)))

print('SVM_Classifier: classification report')
print(classification_report_imbalanced(y_test, y_pred))
print('SVM_Classifier:std_var')
print(np.std(y_test - y_pred))

# Plot ROC-Curve
prec, recall, _ = precision_recall_curve(y_train, y_train_pred)
plt.plot(recall, prec, label = "SVM Classifier")
plt.xlabel('Recall')
plt.ylabel('Precision')
plt.legend()
plt.show()